@@ -1,7 +1,7 @@
 from MEArec.tools import load_templates, load_recordings, save_recording_generator, save_template_generator, \
     get_default_config, plot_templates, plot_recordings, plot_rasters, plot_waveforms, get_templates_features, \
     plot_pca_map, plot_amplitudes, get_default_recordings_params, get_default_templates_params, \
-    get_default_cell_models_folder
+    get_default_cell_models_folder, convert_recording_to_new_version
 from MEArec.generation_tools import gen_recordings, gen_templates, gen_spiketrains
 from MEArec.generators import RecordingGenerator, SpikeTrainGenerator, TemplateGenerator
 from MEArec.simulate_cells import return_bbp_cell, run_cell_model, return_bbp_cell_morphology, \
