@@ -58,11 +58,6 @@ class RecordingGenerator:
           - spike_traces : float (n_spiketrains, n_samples)
     info :  dict
         Info dictionary to instantiate RecordingGenerator with existing data. Same fields as 'params'
-    verbose : bool or int
-        When verbose is 0 or False: no verbose
-        When verbose is 1 or True: main verbose but no verbose each chunk
-        When verbose is 2 : full verbose even each chunk
-        
     """
 
     def __init__(self, spgen=None, tempgen=None, params=None, rec_dict=None, info=None):
@@ -83,6 +78,10 @@ class RecordingGenerator:
                 self.templates = rec_dict['templates']
             else:
                 self.templates = np.array([])
+            if 'original_templates' in rec_dict.keys():
+                self.original_templates = rec_dict['original_templates']
+            else:
+                self.original_templates = np.array([])
             if 'template_locations' in rec_dict.keys():
                 self.template_locations = rec_dict['template_locations']
             else:
@@ -114,7 +113,10 @@ class RecordingGenerator:
             self.info = deepcopy(info)
             self.params = deepcopy(info)
             if len(self.spiketrains) > 0:
-                self.spgen = SpikeTrainGenerator(spiketrains=self.spiketrains, params=self.info['spiketrains'])
+                if 'spiketrains' in self.info:
+                    self.spgen = SpikeTrainGenerator(spiketrains=self.spiketrains, params=self.info['spiketrains'])
+                else:
+                    self.spgen = SpikeTrainGenerator(spiketrains=self.spiketrains, params={'custom': True})
             self.tempgen = None
             if isinstance(self.recordings, np.memmap):
                 self.tmp_mode = 'memmap'
@@ -142,9 +144,6 @@ class RecordingGenerator:
         self._is_tmp_folder_local = False
 
     def __del__(self):
-        self.recordings = None
-        self.spike_traces = None
-
         if not self._is_tmp_folder_local:
             if self.tmp_folder is not None:
                 try:
@@ -163,10 +162,16 @@ class RecordingGenerator:
                 except Exception as e:
                     if self._verbose >= 1:
                         print('Impossible to delete temp file:', fname, 'Error', e)
+        if isinstance(self.recordings, h5py.Dataset):
+            f = self.recordings.file
+            f.close()
+        self.recordings = None
+        self.spike_traces = None
 
     def generate_recordings(self, tmp_mode=None, tmp_folder=None, verbose=None, n_jobs=0):
         """
         Generates recordings
+
         Parameters
         ----------
         tmp_mode : None, 'memmap'
@@ -177,9 +182,9 @@ class RecordingGenerator:
             If None, then it is automatic using tempfile.mkdtemp()
         verbose: bool or int
             Determines the level of verbose. If 1 or True, low-level, if 2 high level, if False, not verbose
-        n_jobs: int if >1 then use joblib to execute chunk in parralel else in loop
-
+        n_jobs: int if >1 then use joblib to execute chunk in parallel else in loop
         """
+
         self.tmp_mode = tmp_mode
         self.tmp_folder = tmp_folder
         self.n_jobs = n_jobs
@@ -364,7 +369,7 @@ class RecordingGenerator:
             n_burst_spikes = params['recordings']['n_burst_spikes']
             if 'max_burst_duration' not in rec_params.keys():
                 params['recordings']['max_burst_duration'] = 100
-            max_burst_duration = 100 * pq.ms
+            max_burst_duration = params['recordings']['max_burst_duration'] * pq.ms
 
             if rec_params['n_bursting'] is None:
                 n_bursting = n_neurons
@@ -667,6 +672,7 @@ class RecordingGenerator:
                 template_rots = np.array(rots)[reordered_idx_cells]
                 template_bin = np.array(bin_cat)[reordered_idx_cells]
                 templates = np.array(eaps)[reordered_idx_cells]
+                self.original_templates = templates
 
                 # find overlapping templates
                 overlapping = find_overlapping_templates(templates, thresh=overlap_threshold)
@@ -812,6 +818,7 @@ class RecordingGenerator:
                         amp, cons = compute_modulation(st, sdrand=sdrand,
                                                        n_spikes=n_burst_spikes, exp=exp_decay,
                                                        max_burst_duration=max_burst_duration)
+
                         amp_mod.append(amp)
                         cons_spikes.append(cons)
                         st.annotate(bursting=True)
@@ -831,6 +838,7 @@ class RecordingGenerator:
                                                        n_spikes=n_burst_spikes, exp=exp_decay,
                                                        max_burst_duration=max_burst_duration)
                         amp_mod.append(amp)
+
                         cons_spikes.append(cons)
                         st.annotate(bursting=True)
                     else:
@@ -1048,7 +1056,7 @@ class RecordingGenerator:
 
         # assign class variables
         params['templates']['overlapping'] = np.array(overlapping)
-        self.recordings = recordings.transpose()
+        self.recordings = recordings
         self.timestamps = timestamps
         self.channel_positions = mea_pos
         self.templates = np.squeeze(templates)
@@ -1057,7 +1065,7 @@ class RecordingGenerator:
         self.template_celltypes = template_celltypes
         self.spiketrains = spiketrains
         self.voltage_peaks = voltage_peaks
-        self.spike_traces = spike_traces.transpose()
+        self.spike_traces = spike_traces
         self.info = params
 
         #############################
