@@ -7,7 +7,6 @@ The function compile_all_mechanisms must be run once before any cell simulation
 """
 
 import os
-from os.path import join
 import sys
 from glob import glob
 import numpy as np
@@ -23,6 +22,23 @@ else:
     use_loader = False
 
 
+def import_LFPy_neuron():
+    try:
+        import LFPy
+    except:
+        raise ModuleNotFoundError("LFPy is not installed. Install it with 'pip install LFPy'")
+
+    try:
+        import neuron
+    except:
+        raise ModuleNotFoundError("NEURON is not installed. Install it from https://www.neuron.yale.edu/neuron/download")
+
+    if StrictVersion(LFPy.__version__) < StrictVersion('2.1'):
+        raise ImportError("LFPy version must be >= 2.1. To use a previous LFPy version, downgrade MEArec to <= 1.4.1")
+
+    return LFPy, neuron
+
+
 def get_templatename(f):
     """
     Assess from hoc file the templatename being specified within
@@ -51,31 +67,32 @@ def compile_all_mechanisms(cell_folder, verbose=False):
     
     Parameters:
     -----------
-    model : string (optional, default='bbp')
-        Cell model type ('bbp' - Blue Brain Project, i.e. NMC database)
+    cell_folder : str
+        Path to cell folder
     """
+    cell_folder = Path(cell_folder)
+    mod_folder = cell_folder / 'mods'
+    if not mod_folder.is_dir():
+        os.makedirs(str(mod_folder))
 
-    if not os.path.isdir(join(cell_folder, 'mods')):
-        os.mkdir(join(cell_folder, 'mods'))
+    neurons = [f for f in cell_folder.iterdir() if 'mods' not in str(f) and not f.name.startswith('.')]
 
-    neurons = [join(cell_folder, f) \
-               for f in os.listdir(join(cell_folder)) \
-               if f != 'mods']
     if verbose:
         print(neurons)
 
-    for nrn in neurons:
-        for nmodl in glob(join(nrn, 'mechanisms', '*.mod')):
-            while not os.path.isfile(join(cell_folder, 'mods', os.path.split(nmodl)[-1])):
-                if sys.platform == 'win32':
-                    _command = 'copy'
-                else:
-                    _command = 'cp'
-                if verbose:
-                    print('{} {} {}'.format(_command, nmodl, join(cell_folder, 'mods')))
-                os.system('{} {} {}'.format(_command, nmodl, join(cell_folder, 'mods')))
+    for neuron in neurons:
+        for nmodl in (neuron / 'mechanisms').iterdir():
+            if nmodl.suffix == '.mod':
+                while not (cell_folder / 'mods' / nmodl.parts[-1]).is_file():
+                    if sys.platform == 'win32':
+                        _command = 'copy'
+                    else:
+                        _command = 'cp'
+                    if verbose:
+                        print(f"{_command} {nmodl} {cell_folder / 'mods'}")
+                    os.system(f"{_command} {nmodl} {cell_folder / 'mods'}")
     starting_dir = os.getcwd()
-    os.chdir(join(cell_folder, 'mods'))
+    os.chdir(str(cell_folder / 'mods'))
     os.system('nrnivmodl')
     os.chdir(starting_dir)
 
@@ -99,9 +116,7 @@ def return_bbp_cell(cell_folder, end_T, dt, start_T, verbose=False):
     cell : object
         LFPy cell object
     """
-    import mpi4py.MPI
-    import LFPy
-    import neuron
+    LFPy, neuron = import_LFPy_neuron()
     neuron.h.load_file("stdrun.hoc")
     neuron.h.load_file("import3d.hoc")
 
@@ -110,7 +125,7 @@ def return_bbp_cell(cell_folder, end_T, dt, start_T, verbose=False):
     if verbose:
         print("Simulating ", cell_folder)
 
-    neuron.load_mechanisms('../mods')
+    neuron.load_mechanisms(str(Path(cell_folder).parent / 'mods'))
 
     f = open("template.hoc", 'r')
     templatename = get_templatename(f)
@@ -125,7 +140,8 @@ def return_bbp_cell(cell_folder, end_T, dt, start_T, verbose=False):
     f.close()
 
     # get synapses template name
-    f = open(join("synapses", "synapses.hoc"), 'r')
+    synapses_file = str(Path("synapses") / "synapses.hoc")
+    f = open(synapses_file, 'r')
     synapses = get_templatename(f)
     f.close()
 
@@ -138,16 +154,16 @@ def return_bbp_cell(cell_folder, end_T, dt, start_T, verbose=False):
 
     if not hasattr(neuron.h, synapses):
         # load synapses
-        neuron.h.load_file(1, join('synapses', 'synapses.hoc'))
+        neuron.h.load_file(1, synapses_file)
 
     if not hasattr(neuron.h, templatename):
         neuron.h.load_file(1, "template.hoc")
 
-    morphologyfile = os.listdir('morphology')[0]  # glob('morphology\\*')[0]
+    morphologyfile = [f for f in Path('morphology').iterdir()][0]
 
     # Instantiate the cell(s) using LFPy
-    cell = LFPy.TemplateCell(morphology=join('morphology', morphologyfile),
-                             templatefile=join('template.hoc'),
+    cell = LFPy.TemplateCell(morphology=str(morphologyfile),
+                             templatefile=str(Path('template.hoc').absolute()),
                              templatename=templatename,
                              templateargs=0,
                              tstop=end_T,
@@ -179,17 +195,18 @@ def return_bbp_cell_morphology(cell_name, cell_folder, pt3d=False):
     cell : object
         LFPy cell object
     """
-    import mpi4py.MPI
-    import LFPy
+    LFPy, neuron = import_LFPy_neuron()
+    cell_folder = Path(cell_folder)
 
-    if not os.path.isdir(join(cell_folder, cell_name)):
-        raise NotImplementedError('Cell model %s is not found in %s' \
-                                  % (cell_name, cell_folder))
+    if not (cell_folder / cell_name).is_dir():
+        raise NotImplementedError(f'Cell model {cell_name} is not found in {cell_folder}')
 
-    morphologyfile = os.listdir(join(cell_folder, cell_name, 'morphology'))[0]
-    morphology = join(cell_folder, cell_name, 'morphology', morphologyfile)
+    morphology_files = [f for f in (cell_folder / cell_name / 'morphology').iterdir()]
+    if len(morphology_files) > 1:
+        raise Exception(f"More than 1 morphology file found for cell {cell_name}")
+    morphology = morphology_files[0]
 
-    cell = LFPy.Cell(morphology=morphology, pt3d=pt3d)
+    cell = LFPy.Cell(morphology=str(morphology), pt3d=pt3d)
     return cell
 
 
@@ -295,6 +312,8 @@ def run_cell_model(cell_model_folder, verbose=False, sim_folder=None, save=True,
 
     """
     cell_name = Path(cell_model_folder).parts[-1]
+    if sim_folder is not None:
+        sim_folder = Path(sim_folder)
 
     if custom_return_cell_function is None:
         return_function = return_bbp_cell
@@ -303,14 +322,14 @@ def run_cell_model(cell_model_folder, verbose=False, sim_folder=None, save=True,
 
     if save:
         assert sim_folder is not None, "Specify 'save_sim_folder' argument!"
-        if not os.path.isdir(sim_folder):
-            os.makedirs(sim_folder)
+        if not sim_folder.is_dir():
+            os.makedirs(str(sim_folder))
 
-        imem_files = [f for f in os.listdir(sim_folder) if 'imem' in f]
-        vmem_files = [f for f in os.listdir(sim_folder) if 'vmem' in f]
+        imem_files = [f for f in sim_folder.iterdir() if 'imem' in f.name]
+        vmem_files = [f for f in sim_folder.iterdir() if 'vmem' in f.name]
 
-        if not (np.any([cell_name in ifile for ifile in imem_files]) and
-                np.any([cell_name in vfile for vfile in vmem_files])):
+        if not (np.any([cell_name in ifile.name for ifile in imem_files]) and
+                np.any([cell_name in vfile.name for vfile in vmem_files])):
 
             np.random.seed(kwargs['seed'])
             T = kwargs['sim_time'] * 1000
@@ -364,10 +383,10 @@ def run_cell_model(cell_model_folder, verbose=False, sim_folder=None, save=True,
                 i_spikes[idx, :, :] = i_spike
                 v_spikes[idx, :] = v_spike
 
-            if not os.path.isdir(sim_folder):
-                os.makedirs(sim_folder)
-            np.save(join(sim_folder, 'imem_%d_%s.npy' % (num_spikes - 1, cell_name)), i_spikes)
-            np.save(join(sim_folder, 'vmem_%d_%s.npy' % (num_spikes - 1, cell_name)), v_spikes)
+            if not sim_folder.is_dir():
+                os.makedirs(str(sim_folder))
+            np.save(str(sim_folder / f'imem_{num_spikes - 1}_{cell_name}.npy'), i_spikes)
+            np.save(str(sim_folder / f'vmem_{num_spikes - 1}_{cell_name}.npy'), v_spikes)
 
         else:
             if verbose:
@@ -444,7 +463,7 @@ def calculate_extracellular_potential(cell, mea, ncontacts=10, position=None, ro
     v_ext : np.array
         Extracellular potential computed on the electrodes (n_elec, n_timestamps)
     '''
-    import LFPy
+    LFPy, neuron = import_LFPy_neuron()
 
     if isinstance(mea, str):
         mea_obj = mu.return_mea(mea)
@@ -455,38 +474,10 @@ def calculate_extracellular_potential(cell, mea, ncontacts=10, position=None, ro
     else:
         raise Exception("")
 
-    pos = mea_obj.positions
-    elinfo = mea_obj.info
-
-    elec_x = pos[:, 0]
-    elec_y = pos[:, 1]
-    elec_z = pos[:, 2]
-
-    N = np.empty((pos.shape[0], 3))
-    for i in np.arange(N.shape[0]):
-        N[i] = [1, 0, 0]  # normal vec. of contacts
-
-    # Add square electrodes (instead of circles)
     if ncontacts > 1:
-        electrode_parameters = {
-            'sigma': 0.3,  # extracellular conductivity
-            'x': elec_x,  # x,y,z-coordinates of contact points
-            'y': elec_y,
-            'z': elec_z,
-            'n': ncontacts,
-            'r': elinfo['size'],
-            'N': N,
-            'contact_shape': elinfo['shape']
-        }
+        electrodes = LFPy.RecExtElectrode(cell, probe=mea_obj, n=ncontacts)
     else:
-        electrode_parameters = {
-            'sigma': 0.3,  # extracellular conductivity
-            'x': elec_x,  # x,y,z-coordinates of contact points
-            'y': elec_y,
-            'z': elec_z
-        }
-
-    electrodes = LFPy.RecExtElectrode(cell, **electrode_parameters)
+        electrodes = LFPy.RecExtElectrode(cell, probe=mea_obj)
 
     if position is not None:
         assert len(position) == 3, "'position' should be a 3d array"
@@ -535,14 +526,16 @@ def calc_extracellular(cell_model_folder, load_sim_folder, save_sim_folder=None,
     --------
         nothing, but saves the result
     """
-    cell_name = os.path.split(cell_model_folder)[-1]
+    LFPy, neuron = import_LFPy_neuron()
+    cell_model_folder = Path(cell_model_folder)
+    cell_name = cell_model_folder.parts[-1]
     cell_save_name = cell_name
+    load_sim_folder = Path(load_sim_folder)
     np.random.seed(seed)
 
     T = kwargs['sim_time'] * 1000
     dt = kwargs['dt']
     rotation = kwargs['rot']
-    print(rotation)
     nobs = kwargs['n']
     ncontacts = kwargs['ncontacts']
     overhang = kwargs['overhang']
@@ -573,10 +566,10 @@ def calc_extracellular(cell_model_folder, load_sim_folder, save_sim_folder=None,
     cell = return_function(cell_model_folder, end_T=T, dt=dt, start_T=0)
 
     # Load data from previous cell simulation
-    imem_file = [f for f in os.listdir(load_sim_folder) if cell_name in f and 'imem' in f][0]
-    vmem_file = [f for f in os.listdir(load_sim_folder) if cell_name in f and 'vmem' in f][0]
-    i_spikes = np.load(join(load_sim_folder, imem_file))
-    v_spikes = np.load(join(load_sim_folder, vmem_file))
+    imem_file = [f for f in load_sim_folder.iterdir() if cell_name in f.name and 'imem' in f.name][0]
+    vmem_file = [f for f in load_sim_folder.iterdir() if cell_name in f.name and 'vmem' in f.name][0]
+    i_spikes = np.load(str(imem_file))
+    v_spikes = np.load(str(vmem_file))
     cell.tvec = np.arange(i_spikes.shape[-1]) * dt
 
     save_spikes = []
@@ -591,46 +584,26 @@ def calc_extracellular(cell_model_folder, load_sim_folder, save_sim_folder=None,
     # Create save folder
     if save:
         assert save_sim_folder is not None, "Specify 'save_sim_folder' argument!"
-        sim_folder = join(save_sim_folder, rotation)
-        save_folder = join(sim_folder, 'tmp_%d_%s' % (target_num_spikes, MEAname))
+        save_sim_folder = Path(save_sim_folder)
+        sim_folder = save_sim_folder / rotation
+        save_folder = sim_folder / f'tmp_{target_num_spikes}_{MEAname}'
 
-        if not os.path.isdir(save_folder):
-            os.makedirs(save_folder)
+        if not save_folder.is_dir():
+            os.makedirs(str(save_folder))
 
     if verbose:
-        print('Cell ', cell_save_name, ' extracellular spikes to be simulated')
+        print(f'Cell {cell_save_name} extracellular spikes to be simulated')
 
-    mea = mu.return_mea(info=elinfo)
+    mea = mu.return_mea(MEAname)
+    if ncontacts > 1:
+        electrodes = LFPy.RecExtElectrode(cell, probe=mea, n=ncontacts)
+    else:
+        electrodes = LFPy.RecExtElectrode(cell, probe=mea)
     pos = mea.positions
-
     elec_x = pos[:, 0]
     elec_y = pos[:, 1]
     elec_z = pos[:, 2]
 
-    N = np.empty((pos.shape[0], 3))
-    for i in np.arange(N.shape[0]):
-        N[i] = [1, 0, 0]  # normal vec. of contacts
-
-    # Add square electrodes (instead of circles)
-    if ncontacts > 1:
-        electrode_parameters = {
-            'sigma': 0.3,  # extracellular conductivity
-            'x': elec_x,  # x,y,z-coordinates of contact points
-            'y': elec_y,
-            'z': elec_z,
-            'n': ncontacts,
-            'r': elinfo['size'],
-            'N': N,
-            'contact_shape': elinfo['shape']
-        }
-    else:
-        electrode_parameters = {
-            'sigma': 0.3,  # extracellular conductivity
-            'x': elec_x,  # x,y,z-coordinates of contact points
-            'y': elec_y,
-            'z': elec_z
-        }
-
     if x_lim is None:
         x_lim = [float(np.min(elec_x) - overhang),
                  float(np.max(elec_x) + overhang)]
@@ -656,7 +629,7 @@ def calc_extracellular(cell_model_folder, load_sim_folder, save_sim_folder=None,
 
         if not drifting:
             espikes, pos, rot, offs = return_extracellular_spike(cell=cell, cell_name=cell_name, model_type=model_type,
-                                                                 electrode_parameters=electrode_parameters,
+                                                                 electrodes=electrodes,
                                                                  limits=[x_lim, y_lim, z_lim], rotation=rotation,
                                                                  pos=position)
             # Method of Images for semi-infinite planes
@@ -675,8 +648,10 @@ def calc_extracellular(cell_model_folder, load_sim_folder, save_sim_folder=None,
                           str(len(save_spikes)) + '/' + str(target_num_spikes) + ']')
                 saved += 1
         else:
-            espikes, pos, rot, offs = return_extracellular_spike(cell, cell_name, 'bbp', electrode_parameters,
-                                                                 [x_lim, y_lim, z_lim], rotation, pos=position)
+            espikes, pos, rot, offs = return_extracellular_spike(cell=cell, cell_name=cell_name, model_type=model_type,
+                                                                 electrodes=electrodes,
+                                                                 limits=[x_lim, y_lim, z_lim], rotation=rotation,
+                                                                 pos=position)
 
             # Method of Images for semi-infinite planes
             if elinfo['type'] == 'mea':
@@ -705,9 +680,10 @@ def calc_extracellular(cell_model_folder, load_sim_folder, save_sim_folder=None,
                                 x_lim[0] < x_rand < x_lim[1] and \
                                 y_lim[0] < y_rand < y_lim[1] and \
                                 z_lim[0] < z_rand < z_lim[1]:
-                            espikes, pos, rot_, offs = return_extracellular_spike(cell, cell_name, model_type,
-                                                                                  electrode_parameters,
-                                                                                  [x_lim, y_lim, z_lim],
+                            espikes, pos, rot_, offs = return_extracellular_spike(cell=cell, cell_name=cell_name,
+                                                                                  model_type=model_type,
+                                                                                  electrodes=electrodes,
+                                                                                  limits=[x_lim, y_lim, z_lim],
                                                                                   rotation=None,
                                                                                   pos=final_pos)
                             # check final position spike amplitude
@@ -731,9 +707,10 @@ def calc_extracellular(cell_model_folder, load_sim_folder, save_sim_folder=None,
                         drift_dir = np.array(final_pos) - np.array(init_pos)
                         for i, dp in enumerate(np.linspace(0, 1, drift_steps)):
                             pos_drift = init_pos + dp * drift_dir
-                            espikes, pos, r_, offs = return_extracellular_spike(cell, cell_name, model_type,
-                                                                                electrode_parameters,
-                                                                                [x_lim, y_lim, z_lim],
+                            espikes, pos, r_, offs = return_extracellular_spike(cell=cell, cell_name=cell_name,
+                                                                                model_type=model_type,
+                                                                                electrodes=electrodes,
+                                                                                limits=[x_lim, y_lim, z_lim],
                                                                                 rotation=None,
                                                                                 pos=pos_drift)
                             espikes = center_espike(espikes, cut_out)
@@ -773,9 +750,9 @@ def calc_extracellular(cell_model_folder, load_sim_folder, save_sim_folder=None,
     save_rot = np.array(save_rot)
 
     if save:
-        np.save(join(save_folder, 'eap-%s' % cell_save_name), save_spikes)
-        np.save(join(save_folder, 'pos-%s' % cell_save_name), save_pos)
-        np.save(join(save_folder, 'rot-%s' % cell_save_name), save_rot)
+        np.save(str(save_folder / f'eap-{cell_save_name}'), save_spikes)
+        np.save(str(save_folder / f'pos-{cell_save_name}'), save_pos)
+        np.save(str(save_folder / f'rot-{cell_save_name}'), save_rot)
     else:
         return save_spikes, save_pos, save_rot
 
@@ -901,7 +878,7 @@ def get_physrot_specs(cell_name, model):
 
 
 def return_extracellular_spike(cell, cell_name, model_type,
-                               electrode_parameters, limits, rotation, pos=None):
+                               electrodes, limits, rotation, pos=None):
     """    Calculate extracellular spike on MEA 
            at random position relative to cell
 
@@ -911,8 +888,8 @@ def return_extracellular_spike(cell, cell_name, model_type,
         cell object from LFPy
     cell_name: string
         name of cell model
-    electrode_parameters: dict
-        parameters to initialize LFPy.RecExtElectrode
+    electrodes: LFPyRecExtElectrode
+        The LFPy electrode object
     limits: array_like
         boundaries for neuron locations, shape=(3,2)
     rotation: string 
@@ -924,7 +901,6 @@ def return_extracellular_spike(cell, cell_name, model_type,
     --------
     Extracellular spike for each MEA contact site
     """
-    import LFPy
 
     def get_xyz_angles(R):
         """ Get rotation angles for each axis from rotation matrix
@@ -997,9 +973,7 @@ def return_extracellular_spike(cell, cell_name, model_type,
         else:
             return False
 
-    electrodes = LFPy.RecExtElectrode(cell, **electrode_parameters)
-
-    """Rotate neuron"""
+    # rotate neuron
     if rotation == 'norot':
         if model_type == 'bbp':
             # orientate cells in z direction
@@ -1179,7 +1153,7 @@ if __name__ == '__main__':
         rot = params['rot']
 
         extra_sim_folder = params['templates_folder']
-        vm_im_sim_folder = join(params['templates_folder'], 'intracellular')
+        vm_im_sim_folder = str(Path(params['templates_folder']) / 'intracellular')
 
         print('Intracellular simulation: ', cell_model)
         run_cell_model(cell_model_folder=cell_model, sim_folder=vm_im_sim_folder, verbose=verbose, **params)
