@@ -4,6 +4,7 @@ import os
 import numpy as np
 import unittest
 import MEArec as mr
+from pathlib import Path
 import tempfile
 import shutil
 import yaml
@@ -27,20 +28,23 @@ class TestGenerators(unittest.TestCase):
     def setUpClass(self):
         info, info_folder = mr.get_default_config()
         cell_models_folder = mr.get_default_cell_models_folder()
-        self.num_cells = len([f for f in os.listdir(cell_models_folder) if 'mods' not in f])
+        self.num_cells = len([f for f in Path(cell_models_folder).iterdir() if 'mods' not in
+                              f.name and not f.name.startswith('.')])
         self.n = 10
         self.n_drift = 5
 
         templates_params = mr.get_default_templates_params()
         rec_params = mr.get_default_recordings_params()
 
-        if not local_temp:
-            self.test_dir = tempfile.mkdtemp()
-        else:
-            self.test_dir = './tmp'
+        # Set seed
+        np.random.seed(2308)
 
-        if not (os.path.isfile(self.test_dir + '/templates.h5')
-                and os.path.isfile(self.test_dir + '/templates_drift.h5')):
+        if not local_temp:
+            self.test_dir = Path(tempfile.mkdtemp())
+        else:
+            self.test_dir = Path('./tmp')
+
+        if not (self.test_dir / 'templates.h5').is_file() and not (self.test_dir / 'templates_drift.h5').is_file():
             templates_params['n'] = self.n
             templates_params['ncontacts'] = 1
             templates_params['probe'] = 'Neuronexus-32'
@@ -65,18 +69,18 @@ class TestGenerators(unittest.TestCase):
             self.num_steps_drift = self.tempgen_drift.templates.shape[1]
 
             print('Making test recordings to test load functions')
-            mr.save_template_generator(self.tempgen, self.test_dir + '/templates.h5')
-            mr.save_template_generator(self.tempgen_drift, self.test_dir + '/templates_drift.h5')
+            mr.save_template_generator(self.tempgen, self.test_dir / 'templates.h5')
+            mr.save_template_generator(self.tempgen_drift, self.test_dir / 'templates_drift.h5')
         else:
-            print(self.test_dir + '/templates.h5')
-            self.tempgen = mr.load_templates(self.test_dir + '/templates.h5', return_h5_objects=False)
-            self.tempgen_drift = mr.load_templates(self.test_dir + '/templates_drift.h5')
+            print(self.test_dir / 'templates.h5')
+            self.tempgen = mr.load_templates(self.test_dir / 'templates.h5', return_h5_objects=False)
+            self.tempgen_drift = mr.load_templates(self.test_dir / 'templates_drift.h5')
             self.templates_params = self.tempgen.info['params']
             self.templates_params_drift = self.tempgen_drift.info['params']
             self.num_steps_drift = self.tempgen_drift.templates.shape[1]
             self.num_templates, self.num_chan, self.num_samples = self.tempgen.templates.shape
 
-        if not os.path.isfile(self.test_dir + '/recordings.h5'):
+        if not (self.test_dir / 'recordings.h5').is_file():
             ne = 2
             ni = 1
             rec_params['spiketrains']['n_exc'] = ne
@@ -90,9 +94,9 @@ class TestGenerators(unittest.TestCase):
 
             self.recgen = mr.gen_recordings(params=rec_params, tempgen=self.tempgen, verbose=False)
             self.recgen.annotate_overlapping_spikes(parallel=False)
-            mr.save_recording_generator(self.recgen, self.test_dir + '/recordings.h5')
+            mr.save_recording_generator(self.recgen, self.test_dir / 'recordings.h5')
         else:
-            self.recgen = mr.load_recordings(self.test_dir + '/recordings.h5', return_h5_objects=False)
+            self.recgen = mr.load_recordings(self.test_dir / 'recordings.h5', return_h5_objects=False)
 
     @classmethod
     def tearDownClass(self):
@@ -209,7 +213,7 @@ class TestGenerators(unittest.TestCase):
 
                         recgen_mod = mr.gen_recordings(params=rec_params, tempgen=self.tempgen, verbose=False)
 
-                        assert recgen_mod.recordings.shape[0] == num_chan
+                        assert recgen_mod.recordings.shape[1] == num_chan
                         assert len(recgen_mod.spiketrains) == n_neurons
                         assert recgen_mod.channel_positions.shape == (num_chan, 3)
                         if j == 1:
@@ -217,7 +221,7 @@ class TestGenerators(unittest.TestCase):
                         else:
                             assert recgen_mod.templates.shape[:3] == (n_neurons, j, num_chan)
                         assert recgen_mod.voltage_peaks.shape == (n_neurons, num_chan)
-                        assert len(recgen_mod.spike_traces) == n_neurons
+                        assert recgen_mod.spike_traces.shape[1] == n_neurons
                         del recgen_mod
 
     def test_gen_recordings_bursting(self):
@@ -246,12 +250,12 @@ class TestGenerators(unittest.TestCase):
         recgen_burst = mr.gen_recordings(params=rec_params, tempgen=self.tempgen, verbose=False)
         recgen_burst.extract_waveforms()
 
-        assert recgen_burst.recordings.shape[0] == num_chan
+        assert recgen_burst.recordings.shape[1] == num_chan
         assert len(recgen_burst.spiketrains) == n_neurons
         assert recgen_burst.channel_positions.shape == (num_chan, 3)
         assert recgen_burst.templates.shape[:3] == (n_neurons, n_jitter, num_chan)
         assert recgen_burst.voltage_peaks.shape == (n_neurons, num_chan)
-        assert len(recgen_burst.spike_traces) == n_neurons
+        assert recgen_burst.spike_traces.shape[1] == n_neurons
         del recgen_burst
 
         rec_params['recordings']['modulation'] = 'template'
@@ -260,12 +264,12 @@ class TestGenerators(unittest.TestCase):
         recgen_burst = mr.gen_recordings(params=rec_params, tempgen=self.tempgen, verbose=False)
         recgen_burst.extract_waveforms()
 
-        assert recgen_burst.recordings.shape[0] == num_chan
+        assert recgen_burst.recordings.shape[1] == num_chan
         assert len(recgen_burst.spiketrains) == n_neurons
         assert recgen_burst.channel_positions.shape == (num_chan, 3)
         assert recgen_burst.templates.shape[:3] == (n_neurons, n_jitter, num_chan)
         assert recgen_burst.voltage_peaks.shape == (n_neurons, num_chan)
-        assert len(recgen_burst.spike_traces) == n_neurons
+        assert recgen_burst.spike_traces.shape[1] == n_neurons
         del recgen_burst
 
     def test_gen_recordings_noise(self):
@@ -306,7 +310,7 @@ class TestGenerators(unittest.TestCase):
                     if noise_color:
                         rec_params['recordings']['filter_cutoff'] = 500
 
-                    assert recgen_noise.recordings.shape[0] == num_chan
+                    assert recgen_noise.recordings.shape[1] == num_chan
                     assert len(recgen_noise.spiketrains) == n_neurons
                     assert recgen_noise.channel_positions.shape == (num_chan, 3)
                     assert len(recgen_noise.spike_traces) == n_neurons
@@ -338,11 +342,9 @@ class TestGenerators(unittest.TestCase):
         rec_params['recordings']['filter'] = False
         recgen_noise = mr.gen_recordings(params=rec_params, tempgen=self.tempgen, verbose=False)
 
-        assert recgen_noise.recordings.shape[0] == num_chan
+        assert recgen_noise.recordings.shape[1] == num_chan
         assert recgen_noise.channel_positions.shape == (num_chan, 3)
         assert len(recgen_noise.spiketrains) == n_neurons
-        assert len(recgen_noise.spiketrains) == n_neurons
-        assert len(recgen_noise.spiketrains) == n_neurons
         assert len(recgen_noise.spike_traces) == n_neurons
         assert np.isclose(np.std(recgen_noise.recordings), noise_level, atol=1)
         del recgen_noise
@@ -350,11 +352,9 @@ class TestGenerators(unittest.TestCase):
         rec_params['recordings']['chunk_duration'] = 1
         recgen_noise = mr.gen_recordings(params=rec_params, tempgen=self.tempgen, verbose=False)
 
-        assert recgen_noise.recordings.shape[0] == num_chan
+        assert recgen_noise.recordings.shape[1] == num_chan
         assert recgen_noise.channel_positions.shape == (num_chan, 3)
         assert len(recgen_noise.spiketrains) == n_neurons
-        assert len(recgen_noise.spiketrains) == n_neurons
-        assert len(recgen_noise.spiketrains) == n_neurons
         assert len(recgen_noise.spike_traces) == n_neurons
         assert np.isclose(np.std(recgen_noise.recordings), noise_level, atol=1)
         del recgen_noise
@@ -402,7 +402,7 @@ class TestGenerators(unittest.TestCase):
                                 rec_params['recordings']['shape_mod'] = True
                             recgen_drift = mr.gen_recordings(params=rec_params, tempgen=self.tempgen_drift,
                                                              verbose=False)
-                            assert recgen_drift.recordings.shape[0] == num_chan
+                            assert recgen_drift.recordings.shape[1] == num_chan
                             assert len(recgen_drift.spiketrains) == n_neurons
                             assert recgen_drift.channel_positions.shape == (num_chan, 3)
                             if j == 1:
@@ -412,13 +412,13 @@ class TestGenerators(unittest.TestCase):
                                 assert recgen_drift.templates.shape[0] == n_neurons
                                 assert recgen_drift.templates.shape[2] == j
                                 assert recgen_drift.templates.shape[3] == num_chan
-                            assert len(recgen_drift.spike_traces) == n_neurons
+                            assert recgen_drift.spike_traces.shape[1] == n_neurons
                             del recgen_drift
 
     def test_save_load_templates(self):
-        tempgen = mr.load_templates(self.test_dir + '/templates.h5', verbose=True)
-        tempgen_drift = mr.load_templates(self.test_dir + '/templates_drift.h5')
-        tempgen_drift_f = mr.load_templates(self.test_dir + '/templates_drift.h5', return_h5_objects=True)
+        tempgen = mr.load_templates(self.test_dir / 'templates.h5', verbose=True)
+        tempgen_drift = mr.load_templates(self.test_dir / 'templates_drift.h5')
+        tempgen_drift_f = mr.load_templates(self.test_dir / 'templates_drift.h5', return_h5_objects=True)
 
         assert np.allclose(tempgen.templates, self.tempgen.templates)
         assert np.allclose(tempgen.locations, self.tempgen.locations)
@@ -429,9 +429,9 @@ class TestGenerators(unittest.TestCase):
         assert np.allclose(tempgen_drift.templates, np.array(tempgen_drift_f.templates))
 
     def test_save_load_recordings(self):
-        recgen_loaded = mr.load_recordings(self.test_dir + '/recordings.h5', return_h5_objects=False, verbose=True, )
-        recgen_loaded_f = mr.load_recordings(self.test_dir + '/recordings.h5', return_h5_objects=False, verbose=True)
-        recgen_loaded_r = mr.load_recordings(self.test_dir + '/recordings.h5', load=['recordings'],
+        recgen_loaded = mr.load_recordings(self.test_dir / 'recordings.h5', return_h5_objects=False, verbose=True, )
+        recgen_loaded_f = mr.load_recordings(self.test_dir / 'recordings.h5', return_h5_objects=False, verbose=True)
+        recgen_loaded_r = mr.load_recordings(self.test_dir / 'recordings.h5', load=['recordings'],
                                              return_h5_objects=True, verbose=True)
 
         assert np.allclose(recgen_loaded.templates, self.recgen.templates)
@@ -493,13 +493,13 @@ class TestGenerators(unittest.TestCase):
         rec_params['recordings']['fs'] = fs
         recgen_rs = mr.gen_recordings(params=rec_params, tempgen=self.tempgen, verbose=False)
 
-        assert recgen_rs.recordings.shape[0] == num_chan
-        assert recgen_rs.recordings.shape[1] == int(duration * fs)
+        assert recgen_rs.recordings.shape[0] == int(duration * fs)
+        assert recgen_rs.recordings.shape[1] == num_chan
         assert recgen_rs.channel_positions.shape == (num_chan, 3)
         assert len(recgen_rs.spiketrains) == n_neurons
         assert len(recgen_rs.spiketrains) == n_neurons
         assert len(recgen_rs.spiketrains) == n_neurons
-        assert len(recgen_rs.spike_traces) == n_neurons
+        assert recgen_rs.spike_traces.shape[1] == n_neurons
         del recgen_rs
 
     def test_recordings_backend(self):
@@ -602,10 +602,10 @@ class TestGenerators(unittest.TestCase):
         info, info_folder = mr.get_default_config()
         cell_models_folder = info['cell_models_folder']
         tempgen = mr.gen_templates(cell_models_folder, params={'n': 2}, templates_tmp_folder=info['templates_folder'])
-        recgen = mr.gen_recordings(templates=self.test_dir + '/templates.h5', verbose=False)
+        recgen = mr.gen_recordings(templates=self.test_dir / 'templates.h5', verbose=False)
         recgen.params['recordings']['noise_level'] = 0
         recgen.generate_recordings()
-        recgen_loaded = mr.load_recordings(self.test_dir + '/recordings.h5', verbose=True)
+        recgen_loaded = mr.load_recordings(self.test_dir / 'recordings.h5', verbose=True)
         recgen_loaded.params['recordings']['noise_level'] = 0
         recgen_loaded.generate_recordings()
         recgen_empty = mr.RecordingGenerator(rec_dict={}, info={})
@@ -622,9 +622,9 @@ class TestGenerators(unittest.TestCase):
         assert np.min(tempgen.locations[:, 0]) > templates_params['xlim'][0] \
                and np.max(tempgen.locations[:, 0]) < templates_params['xlim'][1]
 
-        assert recgen.recordings.shape[0] == self.num_chan
+        assert recgen.recordings.shape[1] == self.num_chan
         assert recgen.channel_positions.shape == (self.num_chan, 3)
-        assert recgen_loaded.recordings.shape[0] == self.num_chan
+        assert recgen_loaded.recordings.shape[1] == self.num_chan
         assert recgen_loaded.channel_positions.shape == (self.num_chan, 3)
         assert len(recgen_empty.recordings) == 0
         del recgen, recgen_empty
@@ -643,7 +643,7 @@ class TestGenerators(unittest.TestCase):
                                      '-nc', '2', '-ov', '20', '-s', '1', '-mind', '10', '-maxd', '100',
                                      '-drst', '10', '-v'])
         assert result.exit_code == 0
-        result = runner.invoke(cli, ["gen-recordings", '-t', self.test_dir + '/templates.h5', '-ne', '2', '-ni', '1',
+        result = runner.invoke(cli, ["gen-recordings", '-t', str(self.test_dir / 'templates.h5'), '-ne', '2', '-ni', '1',
                                      '-fe', '5', '-fi', '15', '-se', '1', '-si', '1', '-mr', '0.2',
                                      '-rp', '2', '-p', 'poisson', '-md', '1', '-mina', '10', '-maxa', '1000',
                                      '--fs', '32000', '-sr', '0', '-sj', '1', '-nl', '10', '-m', 'none',
@@ -679,18 +679,17 @@ class TestGenerators(unittest.TestCase):
         assert result.exit_code == 0
 
     def test_simulate_cell(self):
-        cell_folder = mr.get_default_cell_models_folder()
+        cell_folder = Path(mr.get_default_cell_models_folder())
         params = mr.get_default_templates_params()
 
         target_spikes = [3, 50]
         params['target_spikes'] = target_spikes
-        cells = os.listdir(cell_folder)
-        cell_name = [c for c in cells if 'TTPC1' in c][0]
-        cell_path = os.path.join(cell_folder, cell_name)
+        cell_path = [c for c in cell_folder.iterdir() if 'TTPC1' in c.name][0]
+        cell_name = cell_path.parts[-1]
 
-        cell, v, i = mr.run_cell_model(cell_model_folder=cell_path, sim_folder=None, verbose=True,
+        cell, v, i = mr.run_cell_model(cell_model_folder=str(cell_path), sim_folder=None, verbose=True,
                                        save=False, return_vi=True, **params)
-        c = mr.return_bbp_cell_morphology(cell_name, cell_folder)
+        c = mr.return_bbp_cell_morphology(str(cell_name), cell_folder)
         assert target_spikes[0] <= len(v) <= target_spikes[1]
         assert target_spikes[0] <= len(i) <= target_spikes[1]
         assert len(c.xmid) == len(c.ymid) and len(c.xmid) == len(c.zmid)
